#!/usr/bin/python
# Nom de fichier : bddrequetes.py

import sys
import psycopg2
from pyproj import Proj
from varglob import *

class GestionBD(object):
    """Mise en place et interfacage d'une base de donnees PostgreSQL""" 
    def __init__(self, dbName, user, passwd, host, port =5432):
	"etablissement de la connexion - Creation du curseur" 
        try: 
	    self.baseDonn = psycopg2.connect(host=host, port=port,database=dbName,user=user,password=passwd) 
        except Exception as err: 
            print("La connexion avec la base de donnees a echoue :\nErreur detectee :\n%s" % err) 
            self.echec =1 
        else: 
	    self.cursor = self.baseDonn.cursor()   # creation du curseur 
	    self.echec =0 

    def creerTables(self, dicTables): 
	print("Creation des tables decrites dans le dictionnaire <dicTables>.")
	for table in dicTables:	      # parcours des cles du dictionnaire 
	    req = "CREATE TABLE %s (" % table 
	    pk =''
	    for descr in dicTables[table]: 
		nomChamp = descr[0]	  # libelle du champ a creer 
		tch = descr[1]	 # type de champ a creer 
		clep = descr[2]
		if tch =='int': 
		    typeChamp ='INTEGER' 
		    req = req + "%s %s, " % (nomChamp, typeChamp) 
		elif tch =='float':
		    typeChamp ='FLOAT' 
		    req = req + "%s %s, " % (nomChamp, typeChamp) 
		elif tch =='cdc': 
		    typeChamp ='VARCHAR(25)' 
		    req = req + "%s %s, " % (nomChamp, typeChamp) 
		else:
		    typeChamp ='VARCHAR(%s)' % tch 
		    req = req + "%s %s, " % (nomChamp, typeChamp) 
		if clep =='cle primaire':
		    pk = nomChamp 
	    if pk == '': 
		req = req[:-2] + ")" 
	    else: 
		req = req + "CONSTRAINT %s_pk PRIMARY KEY(%s))" % (pk, pk) 
	    print(req)
	    self.executerReq(req) 

    def supprimerTables(self, dicTables): 
	"Suppression de toutes les tables decrites dans <dicTables>" 
	for table in list(dicTables.keys()): 
	    req ="DROP TABLE %s" % table 
	    self.executerReq(req) 
	    self.commit()		 # transfert -> disque 

    def executerReq(self, req, param =None): 
	#Execution de la requete <req>, avec detection d'erreur eventuelle
	try: 
	    self.cursor.execute(req, param) 
	except Exception as err: 
	# afficher la requete et le message d'erreur systeme : 
	    print "Requete SQL incorrecte :\n{}\nErreur detectee :".format(req),err
	    return 0 
	else:
	    return 1 
    
    def resultatReq(self): 
	"renvoie le resultat de la requete precedente (une liste de tuples)" 
	return self.cursor.fetchall() 
      
    def type_evt(self,evt):
	if self.executerReq("SELECT type FROM evenements WHERE id_evt='%s'"%(evt)):
	    types = self.resultatReq()
	    if types == []:
		res = ''
	    else:
		res = types[0][0]
	return res

    def nom_action(self,evt):
	if self.executerReq("SELECT action FROM evenements WHERE id_evt='%s'"%(evt)):
	    couleurs = self.resultatReq()
	    if couleurs == []:
		res = ''
	    else:
		res = couleurs[0][0]
	return res

    def commit(self): 
	if self.baseDonn: 
	    self.baseDonn.commit()	 # transfert curseur -> disque 

    def close(self): 
	if self.baseDonn: 
	    self.baseDonn.close() 


class Enregistreur(object): 
    # classe pour gerer l'entree d'enregistrements divers
    def __init__(self, bd, table): 
	self.bd =bd 
	self.table =table 
	self.descriptif =Glob.dicoT[table]   # descriptif des champs 

    def entrer(self): 
	#procedure d'entree d'un enregistrement entier
	champs ="("	 # ebauche de chaine pour les noms de champs 
	valeurs =[]	 # liste pour les valeurs correspondantes 
	p = Proj(proj='utm',zone=30,ellps='WGS84') # pour l'ouest de la France, zone UTM30 Nord
	# Demander successivement une valeur pour chaque champ : 
	for cha, type, nom in self.descriptif: 
	    champs = champs + cha + "," 
	    if cha == 'x_utm':
		val = x
	    elif cha == 'y_utm':
		val = y
	    else:
		val = raw_input("Entrez le champ %s :" %nom)
	    if type =="int": 
	    	val =int(val)
	    if cha =='long':
		longitude = val
	    elif cha =='lat':
		latitude = val
		x,y = p(longitude,latitude)
	    valeurs.append(val) 

	balises ="(" + "%s," * len(valeurs)     # balises de conversion 
	champs = champs[:-1] + ")"    # supprimer la derniere virgule, 
	balises = balises[:-1] + ")"  # et ajouter une parenthese 
	req ="INSERT INTO %s %s VALUES %s" % (self.table, champs, balises) 
	self.bd.executerReq(req, valeurs) 

	ch =raw_input("Continuer (O/N) ? ")
	if ch.upper() == "O": 
	    return 0 
	else: 
	    return 1 

    def modifier(self):
	#modification d'un enregistrement
	table='evenements'
	cle_prim='id_evt'
	print "Quel enregistrement voulez-vous modifier ?"
	if self.bd.executerReq("SELECT id_evt FROM %s"%table):
	    resultats = self.bd.resultatReq()
	    print resultats
	evt = raw_input()
	if self.bd.executerReq("SELECT * FROM %s WHERE id_evt='%s'"%(table,evt)):
	    champs = self.bd.resultatReq()
	descr = Glob.dicoT[table]
	print 'Entrez une nouvelle valeur ou laissez vide pour ne pas modifier'
	num = 1
	for cha,type,nom in descr:
	    if cha=='id_evt':
		continue
	    if cha=='x_utm':
		num = num+1
		continue
	    if cha=='y_utm':
		num = num+1
		continue
	    print nom,' : ',champs[0][num]
	    nveau_cha = raw_input("Nouvelle valeur : ")
	    if nveau_cha != '': self.bd.executerReq("UPDATE %s SET %s='%s' WHERE %s='%s'"%(table,cha,nveau_cha,cle_prim,evt))
	    num = num+1

    def afficher(self):
	#afficher une table
	table='evenements'
	if self.bd.executerReq("SELECT * FROM %s"%table):
	    resultats = self.bd.resultatReq()
	    print "Table %s"%table
	    for res in resultats:
		print res
